# This is a minimal reimplementation of the deprecated conda.cli.python_api
# module. It uses the subprocess module to duplicate the fullest extent of
# the run_command functionality needed for tests. But most of the internal
# uses of run_command do not require this overhead and are better served
# working in process. So when subprocess is needed, we do this:
#   from anaconda_auth._conda.conda_api import Commands, run_command
# and when it is not needed, we do this:
#   from anaconda_auth._conda.conda_api import Commands
#   from conda.cli.main import main as run_command


import subprocess


class Commands:
    CLEAN = "clean"
    CONFIG = "config"
    INSTALL = "install"
    LIST = "list"
    REMOVE = "remove"
    SEARCH = "search"


def run_command(*args, use_exception_handler=False):  # type: ignore
    args = ("python", "-m", "conda") + args
    proc = subprocess.run(args, capture_output=True, text=True)
    return proc.stdout, proc.stderr, proc.returncode



"""Definitions for conda plugins.

This file should not be imported directly, but instead the parent package will
conditionally import it in case conda is not installed in the user's environment.

"""

from typing import Iterable
from typing import Optional

from conda import plugins

from anaconda_auth._conda.auth_handler import AnacondaAuthHandler
from anaconda_auth._conda.conda_token import cli

__all__ = ["conda_subcommands", "conda_auth_handlers"]


def _cli_wrapper(argv: Optional[list[str]] = None) -> int:  # type: ignore
    # If argv is empty tuple, we need to set it back to None
    return cli(argv=argv or None)


@plugins.hookimpl
def conda_subcommands() -> Iterable[plugins.CondaSubcommand]:
    """Defines subcommands into conda itself (not `anaconda` CLI)."""
    yield plugins.CondaSubcommand(
        name="token",
        summary="Set repository access token and configure default_channels",
        action=_cli_wrapper,  # type: ignore
    )


@plugins.hookimpl
def conda_auth_handlers() -> Iterable[plugins.CondaAuthHandler]:
    """Defines the auth handler that can be used for specific channels.

    The following shows an example for how to configure a specific channel inside .condarc:

    ```yaml
    channel_settings:
      - channel: https://repo.anaconda.cloud/*
        auth: anaconda-auth
    ```

    """
    yield plugins.CondaAuthHandler(
        name="anaconda-auth",
        handler=AnacondaAuthHandler,
    )
